import os
from google.datacatalog_connectors.rdbms.scrape import query_assembler


class QueryAssembler(query_assembler.QueryAssembler):

    def _get_refresh_statement(self, table_name):
        return "ANALYZE {};".format(table_name)

    def _get_path_to_num_rows_query(self):
        return os.path.join(
            os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
            'config', 'num_rows_query.sql')



#!/usr/bin/python
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import sys

from .scrape import metadata_scraper

from google.datacatalog_connectors.rdbms import \
    datacatalog_cli


class PostgreSQL2DatacatalogCli(datacatalog_cli.DatacatalogCli):

    def _get_metadata_scraper(self):
        return metadata_scraper.MetadataScraper

    def _get_host_arg(self, args):
        return args.postgresql_host

    def _get_connection_args(self, args):
        return {
            'database': args.postgresql_database,
            'host': args.postgresql_host,
            'port': args.postgresql_port,
            'user': args.postgresql_user,
            'pass': args.postgresql_pass
        }

    def _get_entry_group_id(self, args):
        return args.datacatalog_entry_group_id or 'postgresql'

    def _get_metadata_definition_path(self):
        return os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            'config/metadata_definition.json')

    def _get_query_path(self, args):
        return os.path.join(os.path.dirname(os.path.abspath(__file__)),
                            'config/metadata_query.sql')

    def _parse_args(self, argv):
        parser = argparse.ArgumentParser(
            description='Command line to sync postgresql '
            'metadata to Datacatalog')

        parser.add_argument('--datacatalog-project-id',
                            help='Your Google Cloud project ID',
                            required=True)
        parser.add_argument(
            '--datacatalog-location-id',
            help='Location ID to be used for your Google Cloud Datacatalog',
            required=True)
        parser.add_argument('--datacatalog-entry-group-id',
                            help='Entry group ID to be used for your Google '
                            'Cloud Datacatalog')
        parser.add_argument('--datacatalog-entry-resource-url-prefix',
                            help='Entry resource URL prefix '
                            'used in the ingested Data Catalog Entries')
        parser.add_argument(
            '--postgresql-host',
            help='Your postgresql server host, this is required even'
            ' for the raw_metadata_csv,'
            ' so we are able to map the created entries'
            ' resource with the postgresql host',
            required=True)
        parser.add_argument('--postgresql-port',
                            help='Your postgresql server port')
        parser.add_argument('--postgresql-user',
                            help='Your postgresql credentials user')
        parser.add_argument('--postgresql-pass',
                            help='Your postgresql credentials password')
        parser.add_argument('--postgresql-database',
                            help='Your postgresql database name')
        parser.add_argument(
            '--raw-metadata-csv',
            help='Your raw metadata as a csv file, '
            'can be either a local os GCS '
            'path (If supplied ignores the postgresql server credentials)')
        parser.add_argument('--service-account-path',
                            help='Local Service Account path '
                            '(Can be suplied as '
                            'GOOGLE_APPLICATION_CREDENTIALS env '
                            'var)')
        parser.add_argument('--enable-monitoring',
                            help='Enables monitoring metrics on the connector')
        return parser.parse_args(argv)


def main():
    argv = sys.argv
    PostgreSQL2DatacatalogCli().run(argv[1:] if len(argv) > 0 else argv)



#!/usr/bin/python
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.datacatalog_connectors.rdbms.scrape import \
    metadata_scraper
from .query_assembler import QueryAssembler


class MetadataScraper(metadata_scraper.MetadataScraper):

    def _create_rdbms_connection(self, connection_args):
        # import at the method level, because this flow is conditional
        # if the connector reads from a CSV file, this is not used.
        from psycopg2 import connect

        port = connection_args.get('port')

        if port is None:
            port = 5432

        con = connect(database=connection_args['database'],
                      host=connection_args['host'],
                      port=port,
                      user=connection_args['user'],
                      password=connection_args['pass'])
        return con

    def _get_query_assembler(self):
        return QueryAssembler()

    def _execute_refresh_query(self, cursor, query):
        cursor.execute(query)



#!/usr/bin/python
#
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

release_status='Development Status :: 4 - Beta'

with open('README.md') as readme_file:
    readme = readme_file.read()

setuptools.setup(
    name='google-datacatalog-postgresql-connector',
    version='0.9.0',
    author='Google LLC',
    description=
    'Library for ingesting Postgresql metadata into Google Cloud Data Catalog',
    platforms='Posix; MacOS X; Windows',
    packages=setuptools.find_packages(where='./src'),
    namespace_packages=['google', 'google.datacatalog_connectors'],
    package_dir={'': 'src'},
    entry_points={
        'console_scripts': [
            'google-datacatalog-postgresql-connector = google.datacatalog_connectors.postgresql:main',
        ],
    },
    include_package_data=True,
    install_requires=('psycopg2-binary', 'google-datacatalog-rdbms-connector>=0.11.0<0.12.0'),
    setup_requires=('pytest-runner',),
    tests_require=('pytest-cov', 'google-datacatalog-connectors-commons-test>=0.7.0<0.8.0'),
    classifiers=(
        release_status,
        'Programming Language :: Python :: 3.7',
    ),
    long_description=readme,
    long_description_content_type='text/markdown',
)
